"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const definition_1 = require("../common/definition");
const commonFlags = require("../common/flags");
class Info extends command_1.Command {
    async run() {
        const { args, flags } = this.parse(Info);
        const { dereference, bundle, validate, header } = flags;
        const definition = (0, definition_1.resolveDefinition)(args.definition);
        if (!definition) {
            this.error('Please load a definition file', { exit: 1 });
        }
        let document;
        try {
            document = await (0, definition_1.parseDefinition)({
                definition,
                dereference,
                bundle,
                validate,
                servers: flags.server,
                inject: flags.inject,
                header,
            });
        }
        catch (err) {
            this.error(err, { exit: 1 });
        }
        this.log(`Loaded: ${definition}`);
        this.log();
        (0, definition_1.printInfo)(document, this);
        if (flags.operations) {
            this.log();
            this.printOperations(document);
        }
        else {
            this.log();
            this.log(`operations: ${(0, definition_1.getOperations)(document).length}`);
            this.log(`tags: ${document.tags ? document.tags.length : 0}`);
        }
        if (flags.schemas) {
            this.log();
            this.printSchemas(document);
        }
        else {
            this.log(`schemas: ${document.components && document.components.schemas ? Object.entries(document.components.schemas).length : 0}`);
        }
    }
    printOperations(document) {
        const operations = {};
        if (document.tags) {
            for (const tag of document.tags) {
                const { name, description } = tag;
                operations[name] = {
                    description,
                    routes: [],
                };
            }
        }
        for (const path in document.paths) {
            if (document.paths[path]) {
                for (const method in document.paths[path]) {
                    if (document.paths[path][method]) {
                        const { operationId, summary, description, tags } = document.paths[path][method];
                        let route = `${method.toUpperCase()} ${path}`;
                        if (summary) {
                            route = `${route} - ${summary}`;
                        }
                        else if (description) {
                            route = `${route} - ${description}`;
                        }
                        if (operationId) {
                            route = `${route} (${operationId})`;
                        }
                        for (const tag of tags || ['default']) {
                            if (!operations[tag]) {
                                operations[tag] = { routes: [] };
                            }
                            operations[tag].routes.push(route);
                        }
                    }
                }
            }
        }
        this.log(`Operations (${(0, definition_1.getOperations)(document).length}):`);
        for (const tag in operations) {
            if (operations[tag]) {
                const routes = operations[tag].routes;
                for (const route of routes) {
                    this.log(`- ${route}`);
                }
            }
        }
    }
    printSchemas(document) {
        const schemas = (document.components && document.components.schemas) || {};
        const count = Object.entries(schemas).length;
        if (count > 0) {
            this.log(`Schemas (${count}):`);
            for (const schema in schemas) {
                if (schemas[schema]) {
                    this.log(`- ${schema}`);
                }
            }
        }
    }
}
exports.default = Info;
Info.description = 'Display API information';
Info.examples = [
    '$ openapi info https://raw.githubusercontent.com/OAI/OpenAPI-Specification/master/examples/v3.0/petstore.yaml',
    `$ openapi info ./openapi.yml`,
];
Info.flags = {
    ...commonFlags.help(),
    ...commonFlags.parseOpts(),
    operations: command_1.flags.boolean({ description: 'list operations in document', default: false }),
    schemas: command_1.flags.boolean({ description: 'list schemas in document', default: false }),
};
Info.args = [
    {
        name: 'definition',
        description: 'input definition file',
    },
];
